﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Extensions.Collections;       //RCB4を制御するとき使うので、設定しておく
using Rcb4;                         //


namespace RCB4RefMotionMove
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        //全体で使うので、メンバ変数として宣言
        private Config config = new Config();


        ///コンフィグデータを読み込む関数
        private void GetConfigData()
        {
            ByteList cmd = new ByteList();
            byte[] rx = new byte[5];
            //コマンドの生成
            cmd.Bytes = Command.MoveRamToCom(Rcb4.Rcb4.ConfigRamAddress, 2);
            if (Command.Synchronize(serialPort1, cmd.Bytes, ref rx) == true)
            {
                config[0] = rx[2];
                config[1] = rx[3];
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                serialPort1.Open();             //COMポートを開く
            }
            catch
            {

                MessageBox.Show("COMが開けません！！");
                //  return;                         //失敗したらエラーを出して終わる
                serialPort1.Close();

                //Application.Exit();
                this.Close();
            }

            //COMが開くことができたらconfigを呼んでおく
            GetConfigData();

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            serialPort1.Close();
        }


        /// <summary>
        /// モーションを一時停止させるコマンドの発行
        /// </summary>
        private void MotionSuspend()
        {
            ByteList cmd =new ByteList();
            Byte[] rx = new  Byte[4];       //読み取るために必要だが、特に使わない

            //コマンドの生成
            cmd.Bytes = Rcb4.Rcb4.Suspend(config);

            //コマンドの送信
            Command.Synchronize(serialPort1,cmd.Bytes,ref rx);

        }
        /// <summary>
        /// プログラムカウンタのリセット
        /// </summary>
        private void PgCounterReset()
        {
            ByteList cmd = new ByteList();
            Byte[] rx = new Byte[4];       //読み取るために必要だが、特に使わない
           
            //コマンドの生成
            cmd.Bytes = Rcb4.Rcb4.ResetProgramCounter(Rcb4.Rcb4.MainLoopCmd);
            
            //コマンドの送信
            Command.Synchronize(serialPort1, cmd.Bytes, ref rx);
        }

        private void MotionAddressSet(int motionNum)
        {
            ByteList cmd = new ByteList();
            Byte[] rx = new Byte[4];       //読み取るために必要だが、特に使わない

            //モーションのアドレスは＝モーションの先頭アドレス＋モーション番号＊１モーション１つあたりの容量
            int address = (motionNum - 1) * Rcb4.Rcb4.MotionSingleDataCount + Rcb4.Rcb4.MotionRomAddress;

            //コマンドの生成
            cmd.Bytes = Rcb4.Rcb4.Play(address);

            //コマンドの送信
            Command.Synchronize(serialPort1, cmd.Bytes, ref rx);
        }

        private void MotionRestart()
        {
            ByteList cmd = new ByteList();
            Byte[] rx = new Byte[4];       //読み取るために必要だが、特に使わない

            //コマンドの生成
            cmd.Bytes = Rcb4.Rcb4.Resume(config);

            //コマンドの送信
            Command.Synchronize(serialPort1, cmd.Bytes, ref rx);
        }


        private void MotionPlay(int motionNum,int waitTime)
        {
            //モーションを一時中断する（サスペンド）
            MotionSuspend();
            //プログラムカウンタとＥＥＰＲＯＭ更新フラグリセット(プログラムカウンタをリセット）
            PgCounterReset();
            //コール命令で移動先アドレスをセット
            MotionAddressSet(motionNum);
            //モーションをリスタート
            MotionRestart();

            MySleep(waitTime);    //モーションが終わるまで待つ
        }


        /// <summary>
        /// 自作sleep関数
        /// </summary>
        /// <param name="ms"></param>
        private void MySleep(int ms)
        {
            System.Diagnostics.Stopwatch sw = new System.Diagnostics.Stopwatch();

            sw.Start();

            ////
            ////    計測中はイベントを発生しているかどうか見に行く
            ////

            for (; ; )
            {
                if (sw.ElapsedMilliseconds > ms)
                {
                    break;
                }
                Application.DoEvents();

            }
            sw.Stop();
            sw.Reset();

        }


        /// <summary>
        /// モーションボタンを押された時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            MotionPlay(1, 1000);
      
                for (int j = 0; j < 3; j++)
                {
                    MotionPlay(13, 4000);
                }
         /*       for (int k = 0; k < 5; k++)
                {
                    MotionPlay(4, 500);
                }
*/
            
            MotionPlay(1, 1000);
        }

    }
}
